package afd.namesandnumbers;

import java.util.ArrayList;

import afd.nn.Engine;
import android.app.AlertDialog;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.widget.ListView;

public class DetailsView extends SharedAfdActivity{

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.detail_form);
		
		Intent resultsIntent = getIntent();
		String recordKey = resultsIntent.getStringExtra("RecordKey");
		
		// Setup afd.nn.Engine class;
		Engine engine = new Engine(Common.dataPath(this.getApplicationContext()));
		engine.clear();
		int retVal = engine.jumpToRecord(recordKey);
		
		// Deal with any error.
		if (retVal < 0) {
			AlertDialog.Builder dlgAlert = new AlertDialog.Builder(DetailsView.this);
			dlgAlert.setTitle("Error");
			dlgAlert.setMessage(Engine.errorText(retVal));
			dlgAlert.create().show();
			return;
		}
		
		// Retrieve the result
		ArrayList<AddressDetail> details = new ArrayList<AddressDetail>();
		
		AddressDetail field = new AddressDetail();
		field.fieldName = "Organisation";
		field.fieldValue = engine.getOrganisation();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Name";
		field.fieldValue = engine.getName();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Property";
		field.fieldValue = engine.getProperty();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Street";
		field.fieldValue = engine.getStreet();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Locality";
		field.fieldValue = engine.getLocality();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Postcode";
		field.fieldValue = engine.getPostcode();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Latitude";
		field.fieldValue = engine.getLatitude();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "Longitude";
		field.fieldValue = engine.getLongitude();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "NHS Region";
		field.fieldValue = engine.getNHSRegionName();
		details.add(field);
		
		field = new AddressDetail();
		field.fieldName = "NHS Name";
		field.fieldValue = engine.getNHSName();
		details.add(field);
		
		// Display details.
		final ListView lv = (ListView) findViewById(R.id.DetailListView);
		lv.setAdapter(new AddressDetailListAdapter(this, details));
		
		// For testing - Log all fields.
		
	 	Log.i("AFD", "Name: " + engine.getName());
		Log.i("AFD", "Organisation: " + engine.getOrganisation());
		Log.i("AFD", "Property: " + engine.getProperty());
		Log.i("AFD", "Street: " + engine.getStreet());
		Log.i("AFD", "Locality: " + engine.getLocality());
		Log.i("AFD", "Town: " + engine.getTown());
		Log.i("AFD", "PostalCounty: " + engine.getCountyPostal());
		Log.i("AFD", "AbbreviatedPostalCounty: *Not Used*");
		Log.i("AFD", "OptionalCounty: *Not Used*");
		Log.i("AFD", "AbbreviatedOptionalCounty: *Not Used*");
		Log.i("AFD", "CountyTraditional: " + engine.getCountyTraditional());
		Log.i("AFD", "CountyAdministrative: " + engine.getCountyAdministrative());
		Log.i("AFD", "Postcode: " + engine.getPostcode());
		Log.i("AFD", "DPS: " + engine.getDPS());
		Log.i("AFD", "PostcodeFrom: " + engine.getPostcodeFrom());
		Log.i("AFD", "PostcodeType: " + engine.getPostcodeType());
		Log.i("AFD", "MailsortCode: " + engine.getMailsortCode());
		Log.i("AFD", "Phone: " + engine.getPhone());
		Log.i("AFD", "GridE: " + engine.getGridE());
		Log.i("AFD", "GridN: " + engine.getGridN());
		Log.i("AFD", "Miles: *Not Used*");
		Log.i("AFD", "Km: *Not Used*");
		Log.i("AFD", "Latitude: " + engine.getLatitude());
		Log.i("AFD", "Longitude: " + engine.getLongitude());
		Log.i("AFD", "JustBuilt: " + engine.getJustBuilt());
		Log.i("AFD", "UrbanRuralCode: " + engine.getUrbanRuralCode());
		Log.i("AFD", "UrbanRuralName: " + engine.getUrbanRuralName());
		Log.i("AFD", "WardCode: " + engine.getWardCode());
		Log.i("AFD", "WardName: " + engine.getWardName());
		Log.i("AFD", "ConstituencyCode: " + engine.getConstituencyCode());
		Log.i("AFD", "Constituency: " + engine.getConstituencyName());
		Log.i("AFD", "AuthorityCode: " + engine.getAuthorityCode());
		Log.i("AFD", "Authority: " + engine.getAuthorityName());
		Log.i("AFD", "LEACode: " + engine.getLEACode());
		Log.i("AFD", "LEAName: " + engine.getLEAName());
		Log.i("AFD", "TvRegion: " + engine.getTvRegion());
		Log.i("AFD", "Occupancy: " + engine.getOccupancy());
		Log.i("AFD", "OccupancyDescription: *Not Used*");
		Log.i("AFD", "AddressType: " + engine.getAddressType());
		Log.i("AFD", "AddressTypeDescription: *Not Used*");
		Log.i("AFD", "UDPRN: " + engine.getUDPRN());
		Log.i("AFD", "NHSCode: " + engine.getNHSCode());
		Log.i("AFD", "NHSName: " + engine.getNHSName());
		Log.i("AFD", "NHSRegionCode: " + engine.getNHSRegionCode());
		Log.i("AFD", "NHSRegionName: " + engine.getNHSRegionName());
		Log.i("AFD", "PCTCode: " + engine.getPCTCode());
		Log.i("AFD", "PCTName: " + engine.getPCTName());
		Log.i("AFD", "CensationCode: " + engine.getCensationCode());
		Log.i("AFD", "Affluence: " + engine.getAffluence());
		Log.i("AFD", "Lifestage: " + engine.getLifestage());
		Log.i("AFD", "AdditionalCensusInfo: " + engine.getAdditionalCensusInfo());
		Log.i("AFD", "SOALower: *Not Used*");
		Log.i("AFD", "SOAMiddle: *Not Used*");
		Log.i("AFD", "Residency: " + engine.getResidency());
		Log.i("AFD", "HouseholdComposition: " + engine.getHouseholdComposition());
		Log.i("AFD", "Business: " + engine.getBusiness());
		Log.i("AFD", "Size: " + engine.getSize());
		Log.i("AFD", "SICCode: " + engine.getSIC());
		Log.i("AFD", "OnEditedRoll: " + engine.getOnEditedRoll());
		Log.i("AFD", "Gender: " + engine.getGender());
		Log.i("AFD", "Forename: " + engine.getForename());
		Log.i("AFD", "MiddleInitial: " + engine.getMiddleInitial());
		Log.i("AFD", "Surname: " + engine.getSurname());
		Log.i("AFD", "DateOfBirth: " + engine.getDateOfBirth());
		Log.i("AFD", "CouncilTaxBand: " + engine.getCouncilTaxBand());
		Log.i("AFD", "Product: *Not Used*");
		Log.i("AFD", "Key: " + engine.getRecordKey());
		Log.i("AFD", "List: " + engine.getAddressList());
		Log.i("AFD", "Country: *Not Used*");
		Log.i("AFD", "CountryISO: *Not Used*");
		Log.i("AFD", "DXNumber: *Not Used*");
		Log.i("AFD", "DXExchange: *Not Used*");
		Log.i("AFD", "DXProfession: *Not Used*");
		
		Log.i("AFD", "*The following aren't in the PCE standard fields*");
		Log.i("AFD", "STDCode: " + engine.getSTDCode());
		Log.i("AFD", "Distance: " + engine.getDistance());
		Log.i("AFD", "PAFBuilding: " + engine.getPAFBuilding());
		Log.i("AFD", "PAFLocality: " + engine.getPAFLocality());
		Log.i("AFD", "PAFNumber: " + engine.getPAFNumber());
		Log.i("AFD", "PAFStreet: " + engine.getPAFStreet());
		Log.i("AFD", "PAFSubBuilding: " + engine.getPAFSubBuilding());
	}
}
