package afd.namesandnumbers;

import afd.nn.Engine;
import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Build;
import android.widget.Toast;

public class DoFastFindAsyncTask extends AsyncTask<Object, String, Void> {

	private Activity activity;
	private ProgressDialog progressDialog;
	private ResultsCollection results;
	private int retVal;

	Boolean reachedMaxResultLimit;	   // flag indicating that result-set is purposefully truncated.
	final Integer maxResultCount;	   // read in from a resource file.
	Integer resultCount;

	public DoFastFindAsyncTask(Activity fastfindActivity) {
		activity = fastfindActivity;
		maxResultCount = activity.getResources().getInteger(R.integer.search_results_limit);
	}

	@Override
	protected void onPreExecute() {
		progressDialog = new ProgressDialog(activity);
		progressDialog.setMessage("Doing FastFind...");
		progressDialog.setCancelable(true);
		progressDialog.setIndeterminate(true);
		progressDialog.show();
	}

	@Override
	protected Void doInBackground(Object... params) {

		// NOTE this next line fails here because .getText runs on the UI-thread.
		//      Have worked around this by passing in the search string via the params array
		//      BUT could also have simply moved the retrieval to the onPreExecute event...
		// String search = ((FastFindForm)activity).fastfindText.getText().toString();
		String search = (String) params[0];
		results = new ResultsCollection();

		// Setup afd.nn.Engine class;
		String dataPath = (String) params[1];
		Engine engine = new afd.nn.Engine(dataPath);
		engine.clear();
		engine.setSearchLookup(search);

		// Do the lookup.
		retVal = engine.addressGetFirst(Engine.AFD_ALL_RECORDS);
//		adHocDialog(retVal);
		// Deal with any error.
		if (retVal < 0) {
			cancel(true);
			return null;
		}

		reachedMaxResultLimit = false;
		resultCount = 0;

		// Store results.
		//results = new ResultsCollection();

		while (retVal >= 0) {
			if (retVal != Engine.AFD_RECORD_BREAK) {
				String addressList = engine.getAddressList();

				// Remove postcode and spaces from start of AddressList.
				String listItem = addressList.substring(13);
				String recordKey = engine.getRecordKey();

				ResultItem item = new ResultItem();
				item.listValue = listItem;
				item.recordKey = recordKey;

				resultCount++;
				if (resultCount <= maxResultCount) {
					results.add(item);
				} else {
					reachedMaxResultLimit = true;
				}
			}
			retVal = engine.addressGetNext();
		}

		return null;
	}

	protected void adHocDialog(int msg) {
		if (retVal < 0) {
			if (progressDialog != null && progressDialog.isShowing()) {
				progressDialog.dismiss();
			}

			AlertDialog.Builder dlgAlertBuilder = new AlertDialog.Builder(activity);
			dlgAlertBuilder.setTitle("Ad-Hoc Dialog");

			dlgAlertBuilder
					.setMessage("" + msg)
					.setCancelable(false)
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();
						}
					});

			AlertDialog dlgAlert = dlgAlertBuilder.create();
			dlgAlert.show();
		}
	}

	@Override
	protected void onCancelled() {
		if (retVal < 0) {
			if (progressDialog != null && progressDialog.isShowing()) {
				progressDialog.dismiss();
			}

			AlertDialog.Builder dlgAlertBuilder = new AlertDialog.Builder(activity);
			dlgAlertBuilder.setTitle("Error");

			dlgAlertBuilder
					.setMessage(Engine.errorText(retVal))
					.setCancelable(false)
					.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();
						}
					});

			AlertDialog dlgAlert = dlgAlertBuilder.create();
			dlgAlert.show();
		}
	}

	@Override
	protected void onPostExecute(Void result) {
		if (isCancelled() && Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB) {	// onCancelled is only supported since API level 11 (HoneyComb)
			onCancelled();																// so have to invoke it manually for earlier versions.
		} else {
			if (progressDialog != null && progressDialog.isShowing()) {
				progressDialog.dismiss();
			}

			int size;

			try {
				size = results.size();
			} catch (Exception e) {
				size = 0;
			}

			switch (size) {
				case 0:
					AlertDialog.Builder dlgAlert = new AlertDialog.Builder(activity);
					dlgAlert.setTitle("Results");
					dlgAlert.setMessage("No results found.");
					dlgAlert.setCancelable(false);
					dlgAlert.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();
						}
					});
					dlgAlert.create().show();
					break;

				case 1:
					// Go directly to details.
					ResultItem item = results.getList().get(0);
					String recordKey = item.getRecordKey();

					Intent detailsIntent = new Intent(activity, DetailsView.class);
					detailsIntent.putExtra("RecordKey", recordKey);
					activity.startActivity(detailsIntent);
					break;

				default:
					AlertDialog.Builder dlgAlert2 = new AlertDialog.Builder(activity);
					dlgAlert2.setTitle("Results");
					dlgAlert2.setMessage(resultCount + " Results found.");
					dlgAlert2.setCancelable(false);
					dlgAlert2.setPositiveButton("OK", new DialogInterface.OnClickListener() {

						@Override
						public void onClick(DialogInterface dialog, int which) {

							dialog.dismiss();
							Intent resultsIntent = new Intent(activity, ResultsView.class);
							resultsIntent.putExtra("afd.namesandnumbers.ResultsCollection", results);
							try {
								activity.startActivity(resultsIntent);
								if (reachedMaxResultLimit) {
									Toast.makeText(activity.getApplicationContext(),
											"Only the first " + maxResultCount + " matches are displayed.  You may need to refine the search criteria.",
											Toast.LENGTH_LONG).show();
								}
							} catch (Exception e) {
								AlertDialog.Builder dlgAlert3 = new AlertDialog.Builder(activity);
								dlgAlert3.setTitle("Too Many Results");
								dlgAlert3.setMessage(" Unable to display result list.");
								dlgAlert3.setCancelable(false);
								dlgAlert3.setPositiveButton("OK", new DialogInterface.OnClickListener() {

									@Override
									public void onClick(DialogInterface dialog, int which) {
										dialog.dismiss();
									}
								});
								dlgAlert3.create().show();
							}

						}
					});
					dlgAlert2.create().show();

				//	Intent resultsIntent = new Intent(activity, ResultsView.class);
				//	resultsIntent.putExtra("afd.namesandnumbers.ResultsCollection", results);
				//	activity.startActivity(resultsIntent);
			}

		}
	}
}
