package afd.namesandnumbers;

import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;

import afd.nn.*;

public class UpdateForm extends SharedAfdActivity {

	Button checkButton;
	Button downloadButton;
	Button cancelButton;
	Button checkDownloadButton;
	Button applyButton;
	Button saveButton;

	ProgressBar progressBar;	// horizontal bar showing % complete
	ProgressBar busyIndicator;	// rotating busy icon
	TextView busyPct;			// text to overlay busy icon

	EditText licenseText;
	
	ProgressDialog progressDialog;
	int applyUpdate;

	Engine _engine;

	private Engine getEngine(){
		if (_engine == null) {
			AsyncResponse delegate = new AsyncResponse() {
				@Override
				public void processFinish(int output) {
					// this will receive the result fired from the processFinish method of the Engine class
					// IF the activity has been recreated since the download was begun.
					notifyUser(null, checkDownloadButton);
					// return output;
				}
			};

			_engine = new Engine(Common.dataPath(this.getApplicationContext()), delegate, new ProgressIndicator(progressBar), new ProgressIndicator(busyIndicator), new ProgressIndicator(busyPct));
		}
		return _engine;
	}
	private void setEngine(Engine engine){
		_engine = engine;
	}



	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_update_form);
    
		checkButton         = (Button)findViewById(R.id.checkButton);
		downloadButton      = (Button)findViewById(R.id.downloadButton);
		cancelButton        = (Button)findViewById(R.id.cancelButton);
		checkDownloadButton = (Button)findViewById(R.id.checkDownloadButton);
		applyButton         = (Button)findViewById(R.id.applyButton);
		
		licenseText    = (EditText)findViewById(R.id.licenseText);
		saveButton     = (Button)  findViewById(R.id.saveButton);

		progressBar    = (ProgressBar) findViewById(R.id.progressBar);
		busyIndicator  = (ProgressBar) findViewById(R.id.busyIndicator);
		busyPct        = (TextView)    findViewById(R.id.busyPct);

		progressBar.setVisibility(View.INVISIBLE);
		busyIndicator.setVisibility(View.INVISIBLE);
		busyPct.setVisibility(View.INVISIBLE);


		// Setup Engine.
		Engine engine = getEngine();

		// hideProgressIndicator();

		int isComplete = engine.isDownloadComplete();
		switch(isComplete) {
			case 1:
				// Download complete.
				notifyUser("Download complete and ready to apply.", applyButton);
				break;

			case 0:
				// Download in progress.
				progressBar.setVisibility(View.VISIBLE);
				busyIndicator.setVisibility(View.VISIBLE);
				busyPct.setVisibility(View.VISIBLE);

				notifyUser(null, cancelButton);
				break;

			default:
				// Initialise button state.
				checkButton.setVisibility(View.VISIBLE);
				cancelButton.setVisibility(View.GONE);
				downloadButton.setVisibility(View.GONE);
				checkDownloadButton.setVisibility(View.GONE);
				applyButton.setVisibility(View.GONE);

				checkWhetherUpdateIsNeeded(engine);

				break;
		}
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		super.onCreateOptionsMenu(menu);
		menu.findItem(R.id.update_menu_item).setEnabled(false);
		return true;
	}



	public void doCheck(View view){	checkWhetherUpdateIsNeeded(getEngine()); }
	
	public void doDownload(View view){	startUpdateDownload(getEngine());	}

	public void cancelDownload(View view){ 	cancelUpdateDownload(getEngine());	}

	public void checkDownload(View view){ checkDownloadStatus(getEngine());	}




	public void doApply(View view){
		progressDialog = ProgressDialog.show(UpdateForm.this, "", "Applying update...", true, false);
		final Engine engine = getEngine();
		
		new Thread(new Runnable() {
			@Override
			public void run() {
				try{
					applyUpdate = engine.applyUpdate();
					setEngine(engine);

					runOnUiThread(new Runnable() {
						
						@Override
						public void run() {
						// Check update result.
						switch (applyUpdate) {
							case 1:
								// Update complete.
								notifyUser("Update applied successfully.", checkButton);
								applyButton.setEnabled(false);
								setEngine(null);
								progressDialog.cancel();
								break;

							case 0:
								// Download in progress or failed.
								AlertDialog.Builder inprogDlg = new AlertDialog.Builder(UpdateForm.this);
								inprogDlg.setTitle("Download in progress or failed.");

								inprogDlg.setPositiveButton("Continue download", new DialogInterface.OnClickListener() {

									@Override
									public void onClick(DialogInterface dialog, int which) {
										// Wrong button may be displayed.
										notifyUser(null, checkDownloadButton);
									}
								});

								inprogDlg.setNegativeButton("Restart download", new DialogInterface.OnClickListener() {

									@Override
									public void onClick(DialogInterface dialog, int which) {
										progressDialog.cancel();
										doDownload(null);
									}
								});

								inprogDlg.create().show();
								break;

							case -1:
								// downloaded succeeded but write error while unpacking
								notifyUser("Failed to unpack the data files - check available space.", downloadButton);
								setEngine(null);
								progressDialog.cancel();
								break;
						}

						}
					});
				} catch (Exception e) {
					progressDialog.cancel();
					
					// Update failed.
					notifyUser("Update Failed. Try again or Re-download the update", downloadButton);
					setEngine(null);
				}
			}
		}).start();
	}
		
	
	public void saveLicense(View view){
		String license = licenseText.getText().toString();
		int saved = getEngine().saveLicense(license);
		
		if(saved == 0){
			notifyUser("License applied successfully", null);
		} else {
			notifyUser("License not valid", null);
		}
	}


	private void notifyUser(String msg, Button enableButton){
		if (msg != null) {
			AlertDialog.Builder dlgAlert = new AlertDialog.Builder(UpdateForm.this);
			dlgAlert.setTitle("Data Update");
			dlgAlert.setMessage(msg);
			dlgAlert.setCancelable(false);
			dlgAlert.setPositiveButton("OK", new DialogInterface.OnClickListener() {
				
				@Override
				public void onClick(DialogInterface dialog, int which) {
					dialog.dismiss();					
				}
			});
			
			dlgAlert.create().show();
		}
		
		if(enableButton != null){
			// Reset buttons state.
			checkButton.setVisibility(View.GONE);
			cancelButton.setVisibility(View.GONE);
			downloadButton.setVisibility(View.GONE);
			checkDownloadButton.setVisibility(View.GONE);
			applyButton.setVisibility(View.GONE);
	
			// Enable Button.
			enableButton.setVisibility(View.VISIBLE);
			enableButton.setEnabled(true);
		}
	}


	private void checkWhetherUpdateIsNeeded(Engine engine){

		int needUpdate = engine.checkForUpdate();

		switch (needUpdate) {
			case 1: // Update available.
				notifyUser("Data update available.", downloadButton);
				break;

			case 0: // No update available.
				notifyUser("Your data is up to date.", checkButton);
				break;

			case -1: //Error getting data.
				notifyUser("Error checking data.", checkButton);
				break;

			case -7: // License not found.
				notifyUser("Data license error", checkButton);
				break;
		}
	}


	private void startUpdateDownload(Engine engine){
		// NB: although passed as a parameter to the downloadUpdate routine the delegate
		//     will also be used to overwrite the Engine-level reference that was created
		//     onCreate.
		int downloadStarted = engine.downloadUpdate(new AsyncResponse() {
			@Override
			public void processFinish(int output) {
				// this will receive the result fired from the processFinish method of the Engine class
				// IF the activity has NOT been recreated since the download was begun.
				notifyUser(null, checkDownloadButton);
				// return output;
			}
		});

		switch (downloadStarted) {
			case 1: // Download started.
				notifyUser(null, cancelButton);
				applyButton.setEnabled(true);  // QUERY: is this line needed? apply button will be invisible at this point!!!
				break;

			case 0: // No update available.
				notifyUser("Your data is up to date.", checkButton);
				break;

			case -1: // Error getting data.
				notifyUser("Error checking data.", checkButton);
				break;
		}
	}

	private void cancelUpdateDownload(Engine engine) {
		engine.cancelDownloadUpdate();
	//	notifyUser("Download Cancelled.", checkDownloadButton);
	}


	private void checkDownloadStatus(Engine engine){
		int isComplete = engine.isDownloadComplete();

		switch (isComplete) {
			case 1: // Download complete and ready to apply.
				notifyUser("Download complete and ready to apply.", applyButton);
				break;

			case 0: // Download in progress.
				notifyUser("Download in progress.", cancelButton);	// NB: shouldn't be able to reach this
				break;

			case -2: // Download failed.
				notifyUser("Download Failed. Restart download.", downloadButton);
				break;
		}
	}


}
